#!/bin/sh
# $Id: t-pri-1.sh,v 1.6 2011/06/02 15:12:07 ca Exp $
#
# Copyright (c) 2010 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test priority based scheduling
# approach: restrict smtpc such that it can send mails only very slowly
# (e.g., threads_max=1 or maybe via concurrency limits).
# then send many mails with low priority, and thereafter many mails
# with high priority. the high priority mails must be delivered
# before the rest of the low priority ones.
# additionally, we restrict IQDB in size to see whether that causes
# a problem.
# uses smtpcnf.sh.
# ----------------------------------------
#
test -s stop && exit 1
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh
SRCPRG=${STE}/smtpc2

chkcto smtps COMMERCIAL
chkcto smtps SMI_PRI

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# clean up before running tests
cd ../qmgr || exit 1
rm -f ${MTMAP}

# create directory to pass socket
test -d smtps || mkdir smtps

killit()
{
rm -f ${QMGRCONFMAP} ${MTMAP}
exit 1
}

# currently no map data needed/used
${CAT} > ${QMGRCONF} <<EOF
EOF
# create map
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}
rm -f ${MTMAP}
trap killit 2

# number of messages (low priority)
N=40
# ----------------------------------------
${CAT} >${SMXCNF} <<EOF
smtps { log_level = 14; wait_for_server=4;
	daemon_address = localhost:${SRVPORT};
	path = ../smtps/smtps;
	arguments = "smtps -f ${SMXCNF} ${MTA_SERVER_OPTIONS}";
}
smtpc { log_level = 14; remote_port = ${SNKPORT}; wait_for_server=4;
	connect_only_to = 127.0.0.1;
	threads_max = 1; # low number to delay delivery
	io_timeout=60s;
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF}";
}
qmgr { log_level = 14; wait_for_server=4; wait_for_client=4; 
	IQDB { cache_entries_max = 38; } # see above: N
	IBDB { commit_delay_max=1000;}
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF}";
}
smar { log_level = 14;
	path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
}
EOF

${CAT} > ${MT} <<EOF
b.c	[10.2.1.1]
local.dom	lmtp:
EOF

# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
pri2:	local:
pri10: local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

clearlogs

chkc()
{
  if grep "$@" smtpc.log >/dev/null
  then
    :
  else
    ${VERBOSE} && echo "$0: test ${TEST}: missing $@ in smtpc.log"
    ERR=true
  fi
}

# ----------------------------------------
TEST=1
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
ERR=false
# 
if ${SHELL} ${SD}/smtpcnf.sh -Z smtpc2 -1 -3 lmtpsock -Sa10@b.c -Rpri10@local.dom -c "-s $N -m $N -t $N -M PRI=10" -z -s 0 -E 0 -X 0 -A '' -B '-w 1'
then
  N=10
  if ${SRCPRG} -s $N -m $N -t $N -M PRI=2 -r ${H}:${SRVPORT} -fa2@b.c -Rpri2@local.dom >> ${CLTL} 2>&1
  then
    if ${GREP} "total=$N " ${CLTL} >/dev/null
    then
      # check that pri=2 was delivered before pri=10
      if ${AWK} '
BEGIN { PRI2=0; PRI10=0; }
/pri=2,/ { PRI2=FNR; }
/pri=10,/ { PRI10=FNR; }
END {
  if (PRI2 > PRI10) exit 1;
  else exit 0;
}' ${QML}
      then
        :
      else
        ${VERBOSE} && echo "$0: test ${TEST}: pri2 was delivered after pri10"
        ERR=true
      fi
    else
      ${VERBOSE} && echo "$0: test ${TEST}: missing total=40 in log"
      ERR=true
    fi
  else
    :
  fi
  # wait until everything is delivered
  alldelivered

  sigpids -USR1

  # need to count messages accepted by smtps to check whether all have been
  # delivered by smtpc (could also grep Transactions= in SSL)
  A=`grep -c '250 2.0.0 got it id' $SSL`
  D=`ls -1 ${LMTPDIR}/?/S* | wc -l`
  if test X"$D" = "X" -o X"$A" = "X" -o $D -ne $A
  then
    ${VERBOSE} && echo "$0: test ${TEST}: wrong number of delivered messages: $D"
    ERR=true
  fi

else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
${SD}/stopmts.sh

if ${ERR}
then
  :
else
  ${VERBOSE} && echo "$0: test ${TEST} succeeded"
fi

else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

rm -f ${QMGRCONFMAP} ${MTMAP}

# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
