/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-conf-ip-0.c,v 1.1 2010/03/29 02:45:26 ca Exp $")

#include "sm/string.h"
#include "sm/net.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-prt.h"
#include "sm/sockcnf.h"
#define SM_SOCKCNFDEF 1
#include "sm/sockcnfdef.h"
#include "sm/io.h"

typedef struct
{
	sm_ipaddr_T	 ipaddr;
} ip_T;

sm_conf_definition_T
ip_def[] = {
	{ SM_CONF_DEF_MAGIC, "ip", sm_conf_type_ip,
		offsetof(ip_T, ipaddr), sizeof(sm_ipaddr_T),
		"127.0.0.1", SM_CONF_FLAG_KEEP_DEFAULT
	},

	CONF_SENTINEL
};

static ip_T ip;

static void
print_ip(ip_T *st)
{
	sm_io_fprintf(smioout, "ip=%I;\n", &st->ipaddr);
	sm_io_flush(smioout);
}

static int
process(char const *name, FILE *fp, int show)
{
	sm_conf_T *stream;
	int err;
	char buf[SM_CONF_ERROR_BUFFER_SIZE];
	char const *e = NULL;

	if (((stream = sm_conf_new(name ? name : "*stdin*"))) == NULL) {
		sm_io_fprintf(smioerr, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(stream, name, fp)) != 0) {
		sm_io_fprintf(smioerr, "%s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			sm_io_fprintf(smioerr, "%s\n", e);

		sm_conf_destroy(stream);
		return 2;
	}

	if ((err = sm_conf_scan(stream, ip_def, 0, &ip)) != 0) {
		sm_io_fprintf(smioerr, "(while scanning) %s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));
		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			sm_io_fprintf(smioerr, "%s\n", e);
		sm_conf_destroy(stream);
		return 3;
	}
	print_ip(&ip);
	if (show) {
		err = sm_conf_prt_conf(ip_def, &ip, smioout);
		sm_io_flush(smioout);
	}

#if 0
	sm_conf_destroy(stream);
#endif /* 0 */

	return 0;
}

static void
showversion(void)
{
	sm_io_fprintf(smioerr, "compile time options:\n"
#if MTA_NETINET6
	"MTA_NETINET6\n"
#endif
	);
}

int
main(int ac, char **av)
{
	int ai, c, ret, done, show;

	done = 0;
	show = 0;
	while ((c = getopt(ac, av, "f:sV")) != -1) {
		switch (c) {
		  case 'f':
			ret = process(optarg, NULL, show);
			if (ret != 0)
				return ret;
			done = 1;
			break;
		  case 's':
			show = 1;
			break;
		  case 'V':
			showversion();
			exit(0);
			break;
		}
	}
	ac -= optind;
	av += optind;
	if (done) {
		print_ip(&ip);
		return 0;
	}

	if (ac == 0)
		return process("*stdin*", stdin, show);

	for (ai = 0; ai < ac; ai++) {
		ret = process(av[ai], NULL, show);
		if (ret != 0)
			return ret;
	}
	return 0;
}
