/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: fgetsock.c,v 1.1 2011/04/01 17:20:13 ca Exp $")
#include "sm/limits.h"
#include "sm/assert.h"
#include "sm/magic.h"
#include "sm/io.h"
#include "sm/net.h"

/*
**  SM_IO_GETSOCK -- Read data from file into sockaddr
**
**	Parameters:
**		fp -- the file pointer for the buffer to be read from
**		sockaddr -- sockaddr to fill.
**
**	Returns:
**		usual sm_error code; SM_E_OVFLW_NS
*/

sm_ret_T
sm_io_getsock(sm_file_T *fp, sm_sockaddr_P sockaddr)
{
	sm_ret_T ret;
	int len;
	uint32_t family, port;
	uchar *buf;

	SM_IS_FP(fp);
	SM_REQUIRE(sockaddr != NULL);

	ret = sm_io_fgetuint32(fp, &family);
	if (sm_is_err(ret))
		goto error;
	ret = sm_io_fgetuint32(fp, &port);
	if (sm_is_err(ret))
		goto error;

	sockaddr->sa.sa_family = family;
	switch (sockaddr->sa.sa_family) {
	  case AF_INET:
		len = sizeof(sockaddr->sin.sin_addr);
		buf = (uchar *)&sockaddr->sin.sin_addr;
		sockaddr->sin.sin_port = port;
		break;

#if MTA_NETINET6
	  case AF_INET6:
		len = sizeof(sockaddr->sin6.sin6_addr);
		buf = (uchar *)&sockaddr->sin6.sin6_addr;
		sockaddr->sin6.sin6_port = port;
		break;
#endif
	  default:
		ret = sm_error_perm(SM_EM_NONE, EPROTONOSUPPORT);
		goto error;
	}
	ret = sm_io_fgetn(fp, buf, len);
	if (sm_is_err(ret))
		goto error;

	return SM_SUCCESS;

  error:
	return ret;
}
