/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-log-1.c,v 1.4 2014/03/29 14:56:12 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/test.h"
#include "sm/io.h"
#include "sm/log.h"

static int Verbosity = 0;

sm_logcategory_T lcats[] =
{
	{ "test",	0 },
	{ NULL,		0 }
};

sm_logmodule_T lmods[] =
{
	{ "test",		0 },
	{ NULL,			0 }
};

#define LCAT_TEST		(&lcats[0])
#define LMOD_TEST		(&lmods[0])


static sm_ret_T
test(uint loglevel, uint loops, const char *lstr)
{
	sm_ret_T ret;
	uint u;
	sm_log_ctx_P lctx;
	sm_logconfig_P lcfg;

	ret = sm_log_create(NULL, &lctx, &lcfg);
	SM_TEST(sm_is_success(ret));
	if (sm_is_err(ret)) {
		sm_io_fprintf(smioerr, "sev=ERROR, sm_log_create=%m\n", ret);
		goto error;
	}
	ret = sm_log_setfp_fd(lctx, smiolog, SMIOLOG_FILENO);
	SM_TEST(sm_is_success(ret));
	if (sm_is_err(ret)) {
		sm_io_fprintf(smioerr, "sev=ERROR, sm_log_setfp=%m\n", ret);
		goto error;
	}
	ret = sm_log_setdebuglevel(lctx, loglevel);
	SM_TEST(sm_is_success(ret));
	if (sm_is_err(ret)) {
		sm_io_fprintf(smioerr, "sev=ERROR, sm_log_setdebuglevel=%m\n", ret);
		goto error;
	}
	ret = sm_log_registercategories(lctx, lcats);
	SM_TEST(sm_is_success(ret));
	ret = sm_log_registermodules(lctx, lmods);
	SM_TEST(sm_is_success(ret));

	for (u = 0; u < loops; u++) {
		sm_log_write(lctx, LCAT_TEST, LMOD_TEST, SM_LOG_INFO, loglevel,
			"sev=INFO, func=log_perf, index=%u, whatever=%s", u, lstr);
	}
	sm_log_write(lctx, LCAT_TEST, LMOD_TEST, SM_LOG_INFO, loglevel,
		"sev=INFO, func=%s, index=%u, whatever=%s", SMFCT, u, lstr);

	return ret;

  error:
	return ret;
}

int
main(int argc, char *argv[])
{
	int c;
	uint loglevel, loops;
	char *lstr;

	loglevel = 9;
	loops = 1;
	lstr="rhs";
	while ((c = getopt(argc, argv, "l:r:V")) != -1) {
		switch (c) {
		  case 'l':
			lstr = strdup(optarg);
			break;
		  case 'r':
			loops = atoi(optarg);
			break;
		  case 'V':
			++Verbosity;
			break;
		}
	}
	sm_test_begin(argc, argv, "test log perf");
	argc -= optind;
	argv += optind;
	test(loglevel, loops, lstr);
	return sm_test_end();
}
