/*
 *  simple producer/consumer program using pthreads.
 *  see t-prdcns for a better version.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-thrd-p0.c,v 1.3 2010/01/25 05:45:56 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/memops.h"
#include "sm/heap.h"
/* #include "sm/test.h" */
#include "sm/fxszq.h"

#include <stdio.h>
#include <pthread.h>

/*
*/

struct fxszq_S
{
	uint	 fsq_max;   /* allocated size */
	uint	 fsq_cur;   /* currently used (basically last-first) */
	uint	 fsq_first; /* first index to read */
	uint	 fsq_last;  /* last index to read (first to write) */
	int	*fsq_array; /* array */
};
typedef struct fxszq_S	fxszq_T, *fxszq_P;
fxszq_T q1;

#define WHAT_TERM	0
#define WHAT_CONT	1

#define REPS 5

#define T_QS	256

static int Verbose = 0;
static long reps = 0;
static pthread_t task1, task2;
static bool stop = false;

struct t_ctx_S
{
	int				 t_called;
	pthread_t	 	*t_wakeup;
};
typedef struct t_ctx_S t_ctx_T, *t_ctx_P;

pthread_mutex_t	 q_mutex;
pthread_mutex_t	 t1_mutex, t2_mutex;
pthread_cond_t	 t1_cond, t2_cond;
long q_add, q_rm;

#define SM_TEST(x)	assert(x)
#define SM_TEST_E(x)	assert(x)

static void
t_init(uint sz)
{
	int r;

	SM_TEST(sz != 0);
	if (sz == 0)
		return;
	q1.fsq_array = sm_malloc(sizeof(int) * sz);
	SM_TEST(q1.fsq_array != NULL);
	if (q1.fsq_array == NULL)
		return;
	FSQ_INIT(sz, q1.fsq_max, q1.fsq_cur, q1.fsq_first, q1.fsq_last);
	SM_TEST(FSQ_IS_EMPTY(q1.fsq_cur));
	SM_TEST(!FSQ_IS_FULL(q1.fsq_max, q1.fsq_cur));
	r = pthread_mutex_init(&q_mutex, NULL);
	SM_TEST(r == 0);
}

static bool
add_q(int i)
{
	int r;
	bool e;

	r = pthread_mutex_lock(&q_mutex);
	if (FSQ_IS_FULL(q1.fsq_max, q1.fsq_cur)) {
		r = pthread_mutex_unlock(&q_mutex);
		return true;
	}
	FSQ_APPEND(q1.fsq_max, q1.fsq_last, q1.fsq_cur, q1.fsq_array, i);
	SM_TEST(!FSQ_IS_EMPTY(q1.fsq_cur));
	++q_add;
	e = FSQ_IS_FULL(q1.fsq_max, q1.fsq_cur);
	r = pthread_mutex_unlock(&q_mutex);
	return e;
}

static bool
rm_q(void)
{
	int j, h, r;
	bool e;

	r = pthread_mutex_lock(&q_mutex);
	if (FSQ_IS_EMPTY(q1.fsq_cur)) {
		r = pthread_mutex_unlock(&q_mutex);
		return true;
	}
	j = FSQ_GET(q1.fsq_max, q1.fsq_first, q1.fsq_cur, q1.fsq_array, h);
	++q_rm;
	e = FSQ_IS_EMPTY(q1.fsq_cur);
	r = pthread_mutex_unlock(&q_mutex);
	return e;
}

static void *
fct1(void *ctx)
{
	int i;
	bool e;
	t_ctx_P fctx;

	SM_ASSERT(ctx != NULL);
	fctx = (t_ctx_P) ctx;

	while (1) {
		if (Verbose > 0)
			fprintf(stderr, "fct1=%d, reps=%ld\n", fctx->t_called, reps);
		if (fctx->t_called++ > reps || stop) {
			stop = true;
			return 0;
		}

		for (i = 0; i < 10 && !(e = add_q(i)); i++)
			;
		if (Verbose > 0)
			fprintf(stderr, "fct1: e=%d\n", e);
		if (!stop)
			i = pthread_cond_signal(&t2_cond);
		if (!e)
			continue;

		if (Verbose > 0)
			fprintf(stderr, "fct1: beforelock=%d\n", i);
		i = pthread_mutex_lock(&t1_mutex);
		SM_TEST_E(i == 0);
		if (Verbose > 0)
			fprintf(stderr, "fct1: afterlock=%d\n", i);
		while (FSQ_IS_FULL(q1.fsq_max, q1.fsq_cur)) {
			i = pthread_cond_wait(&t1_cond, &t1_mutex);
			SM_TEST_E(i == 0);
			if (Verbose > 0)
			fprintf(stderr, "fct1: afterwait=%d, full=%d\n", i,
				FSQ_IS_FULL(q1.fsq_max, q1.fsq_cur));
			if (stop) return 0;
		}
		i = pthread_mutex_unlock(&t1_mutex);
		SM_TEST_E(i == 0);
	}

 error:
	return 0;
}

static void *
fct2(void *ctx)
{
	int i;
	bool e;
	t_ctx_P fctx;

	SM_ASSERT(ctx != NULL);
	fctx = (t_ctx_P) ctx;

	while (1) {
		if (Verbose > 0)
			fprintf(stderr, "	fct2=%d, reps=%ld\n", fctx->t_called, reps);
		if (fctx->t_called++ > reps || stop) {
			stop = true;
			return 0;
		}

		for (i = 0; i < 5 && !(e = rm_q()); i++)
			;
		if (Verbose > 0)
			fprintf(stderr, "	fct2: e=%d\n", e);
		if (!stop)
			i = pthread_cond_signal(&t1_cond);
		if (!e)
			continue;

		if (Verbose > 0)
			fprintf(stderr, "	fct2: beforelock=%d\n", i);
		i = pthread_mutex_lock(&t2_mutex);
		SM_TEST_E(i == 0);
		if (Verbose > 0)
			fprintf(stderr, "	fct2: afterlock=%d\n", i);
		while (FSQ_IS_EMPTY(q1.fsq_cur)) {
			i = pthread_cond_wait(&t2_cond, &t2_mutex);
			SM_TEST_E(i == 0);
			if (Verbose > 0)
			fprintf(stderr, "	fct2: afterwait=%d, empty=%d\n", i,
				FSQ_IS_EMPTY(q1.fsq_cur));
			if (stop) return 0;
		}
		i = pthread_mutex_unlock(&t2_mutex);
		SM_TEST_E(i == 0);
	}

 error:
	return 0;
}


static void
testev(uint sz)
{
	int r;
	int ret;
	t_ctx_T tctx1, tctx2;

	t_init(sz);

	r = pthread_mutex_init(&t1_mutex, NULL);
	SM_TEST(r == 0);
	r = pthread_mutex_init(&t2_mutex, NULL);
	SM_TEST(r == 0);
	r = pthread_cond_init(&t1_cond, NULL);
	SM_TEST(r == 0);
	r = pthread_cond_init(&t2_cond, NULL);
	SM_TEST(r == 0);

	tctx1.t_called = 0;
	ret = pthread_create(&task1, NULL, fct1, (void *) &tctx1);
	SM_TEST(ret == 0);
	SM_TEST(task1 != NULL);

	tctx2.t_called = 0;
	ret = pthread_create(&task2, NULL, fct2, (void *) &tctx2);
	SM_TEST(ret == 0);
	SM_TEST(task2 != NULL);

	tctx1.t_wakeup = &task2;
	tctx2.t_wakeup = &task1;

	ret = pthread_join(task1, NULL);
	/* SM_TEST(ret == 0); */
	fprintf(stderr, "join1=%d\n", ret);
	ret = pthread_join(task2, NULL);
	/* SM_TEST(ret == 0); */
	fprintf(stderr, "join2=%d\n", ret);

	fprintf(stderr, "adds=%ld, rms=%ld\n", q_add, q_rm);
}

int
main(int argc, char *argv[])
{
	int c, loops;
	uint sz;

	loops = 16;
	reps = 1;
	sz = 100;
	while ((c = getopt(argc, argv, "l:r:V")) != -1)
	{
		switch (c)
		{
		  case 'l':
			loops = atoi(optarg);
			break;
		  case 'r':
			reps = atoi(optarg);
			break;
		  case 'V':
			Verbose++;
			break;
#if 0
		  default:
			usage(argv[0]);
			return(1);
#endif /* 0 */
		}
	}
/*
	sm_test_begin(argc, argv, "test evthr perf 0");
*/
	testev(sz);
	return 0;
/*
	return sm_test_end();
*/
}
